"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.uWave = void 0;
const WebSocket = require("ws");
const events_1 = require("events");
const node_fetch_1 = require("node-fetch");
const modules_1 = require("./modules");
class uWave {
    constructor(options) {
        // #region modules
        this.modules = {};
        this.options = options;
        this.emitter = new events_1.EventEmitter();
        if (!process.env.UWAVE_API_BASE_URL) {
            process.env.UWAVE_API_BASE_URL = options.apiBaseUrl;
        }
        if (options.authImmediately && options.credentials) {
            const credentials = options.credentials;
            delete this.options.credentials;
            this.auth
                .login(credentials.email, credentials.password)
                .then(() => this.connect());
        }
    }
    get auth() {
        return (this.modules.auth ||
            (this.modules.auth = new modules_1.Auth(this, (jwt, socketToken) => {
                this.jwt = jwt;
                this.socketToken = socketToken;
            })));
    }
    get booth() {
        return this.modules.booth || (this.modules.booth = new modules_1.Booth(this));
    }
    sendChat(message) {
        return this.send({ command: 'sendChat', data: message });
    }
    vote(direction) {
        return this.send({ command: 'vote', data: direction });
    }
    // #region http
    request(method, endpoint, data) {
        const fetchOptions = { method };
        fetchOptions.headers = {};
        if (this.jwt) {
            fetchOptions.headers.Authorization = `JWT ${this.jwt}`;
        }
        let url = process.env.UWAVE_API_BASE_URL + endpoint;
        if (method === 'get' && data) {
            const querystring = querystringify(data);
            if (querystring) {
                url += `?${querystring}`;
            }
        }
        else if (data) {
            fetchOptions.headers['Content-Type'] = 'application/json';
            fetchOptions.body = JSON.stringify(data);
        }
        return node_fetch_1.default(url, fetchOptions).then((res) => res.json());
    }
    get(endpoint, query) {
        return this.request('get', endpoint, query);
    }
    post(endpoint, data) {
        return this.request('post', endpoint, data);
    }
    put(endpoint, data) {
        return this.request('put', endpoint, data);
    }
    patch(endpoint, data) {
        return this.request('patch', endpoint, data);
    }
    delete(endpoint, query) {
        return this.request('delete', endpoint, query);
    }
    // #endregion
    // #region socket
    get isConnected() {
        var _a;
        return ((_a = this.socket) === null || _a === void 0 ? void 0 : _a.readyState) === WebSocket.OPEN;
    }
    once(eventName, listener) {
        return this.emitter.on(eventName, listener);
    }
    on(eventName, listener) {
        return this.emitter.on(eventName, listener);
    }
    off(eventName, listener) {
        return this.emitter.off(eventName, listener);
    }
    connect() {
        return this.connectSocket(this.options.wsConnectionString);
    }
    disconnect() {
        if (this.socket) {
            this.socket.close();
        }
    }
    connectSocket(connectionString) {
        this.socket = new WebSocket(connectionString);
        this.socket.onopen = this.onSocketOpen.bind(this);
        this.socket.onclose = this.onSocketClose.bind(this);
        this.socket.onerror = this.onSocketError.bind(this);
        this.socket.onmessage = this.onSocketMessage.bind(this);
    }
    onSocketOpen( /* event: WebSocket.OpenEvent */) {
        return __awaiter(this, void 0, void 0, function* () {
            this.emit('connected');
            if (!this.socket || !this.isConnected)
                return;
            if (!this.socketToken && this.jwt) {
                this.socketToken = yield this.auth.getSocketToken();
            }
            if (this.socketToken) {
                this.socket.send(this.socketToken);
                // the token is expired after being validated
                delete this.socketToken;
            }
        });
    }
    onSocketClose(event) {
        event.target.removeAllListeners();
        event.target.terminate();
        delete this.socket;
        this.emit('disconnected');
    }
    onSocketError( /* event: WebSocket.ErrorEvent */) {
        this.emit('error');
    }
    onSocketMessage(event) {
        if (event.data === uWave.KEEP_ALIVE_MESSAGE)
            return;
        if (typeof event.data !== 'string')
            return;
        let payload;
        try {
            payload = JSON.parse(event.data);
        }
        catch (err) {
            console.error(err);
            return;
        }
        switch (payload.command) {
            default:
                this.emit(payload.command, payload.data);
                break;
        }
    }
    emit(command, payload) {
        return this.emitter.emit(command, payload);
    }
    send(message) {
        if (!this.socket || this.socket.readyState !== WebSocket.OPEN)
            return;
        return this.socket.send(JSON.stringify(message));
    }
}
exports.uWave = uWave;
// #endregion
uWave.KEEP_ALIVE_MESSAGE = '-';
const querystringify = (obj = {}, keyPrefix) => Object.entries(obj)
    .map(([key, value]) => typeof value !== 'object'
    ? [keyPrefix ? `${keyPrefix}[${key}]` : key, value]
        .map(encodeURIComponent)
        .join('=')
    : querystringify(value, key))
    .join('&');
//# sourceMappingURL=index.js.map